import json
import unittest
from bottle import HTTPResponse, request, post, tob, HTTPError
from io import BytesIO
import mock
import rest


class TestPostSeed(unittest.TestCase):
    """
    Tests for the rest interface of the database wrapper.
    Using
    http://stackoverflow.com/questions/28857520/unit-testing-bottle-py-application-that-uses-request-body-results-in-keyerror

    Bottle's suggestions for how to database.
    http://bottlepy.org/docs/dev/recipes.html

    We're using unittest from python standard library.
    """

    def setUp(self):
        # Flush any cached values
        request.bind({})

    def test_create_empty_post(self):
        """
        Tests that the put/create method returns a 204 (no content) when it creates an empty object.
        :return:
        """
        body = '{"name":""}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        r = rest.post_seed()
        self.assertEquals(r.status_code, 204)

    def test_not_json(self):
        """
        Tests that the put/create method returns a 400(no content) when it gets non-json.
        :return:
        """
        body = '{"name":""}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/text"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        try:
            rest.post_seed()
        except HTTPError as e:
            self.assertEquals(e.status_code, 400)

    def test_not_space(self):
        """
        Tests that the put/create method returns a 400(no content) when it gets non-json.
        :return:
        """
        body = '{"name":"a new seed"}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        try:
            rest.post_seed()
        except HTTPError as e:
            self.assertEquals(e.status_code, 400)
            self.assertEquals(e.body,"Only Alpha-Numeric Permitted in Seed Names")

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.add_seed_to_database', return_value=True)
    def test_create_new(self, add_seed_to_db_function):
        """
        Tests that the post/create method returns appropriately when it creates a new object.
        :return:
        """

        body = '{"name":"Weather"}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)

        r = rest.post_seed()
        self.assertEquals(r.status_code, 201)
        self.assertEquals(r.headers['location'][-7:], "Weather")

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.add_seed_to_database', return_value=False)
    def test_create_already_exists(self, add_seed_to_db_function):
        """
        Tests that the post/create method returns appropriately when it creates a new object.
        :return:
        """

        rest.DBWrapper.__pepp_talk_db_wrapper = type("", (), {"add_seed_to_database": (lambda self, **kwargs: False)})()
        body = '{"name":"Weather"}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)

        r = rest.post_seed()
        self.assertEquals(r.status_code, 200)
        self.assertEquals(r.headers['location'][-7:], "Weather")


class TestPostData(unittest.TestCase):
    """
    Tests for the rest interface of the database wrapper.

    Testing the method Post_Data
    Using
    http://stackoverflow.com/questions/28857520/unit-testing-bottle-py-application-that-uses-request-body-results-in-keyerror

    Bottle's suggestions for how to database.
    http://bottlepy.org/docs/dev/recipes.html

    We're using unittest from python standard library.
    """

    def setUp(self):
        # Flush any cached values
        request.bind({})

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.database_has_seed', return_value=False)
    def test_create_empty(self, mock_fun):
        """
        Tests that the put/create method returns a 204 (no content) when it creates an empty object.
        :return:
        """
        body = '{"name":""}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        try:
            rest.post_seed_data("")
        except HTTPResponse as e:
            self.assertEquals(e.status_code, 303)
            self.assertEquals(e.headers['location'][-5:], "Seeds")

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.database_has_seed', return_value=False)
    def test_not_json(self, mock_fun):
        """
        Tests that the put/create method returns a 400(no content) when it gets non-json.
        :return:
        """
        body = '{"name":""}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/text"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        try:
            rest.post_seed_data("")
        except HTTPError as e:
            self.assertEquals(e.status_code, 400)

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.database_has_seed', return_value=True)
    def test_name_doesnt_match(self, mock_fun):
        """
        Tests that the put/create method returns a 400(no content) when it gets non-json.
        :return:
        """
        body = '{"name":"Weather"}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        try:
            rest.post_seed_data("Finance")
        except HTTPError as e:
            self.assertEquals(e.status_code, 400)

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.add_data_to_database', return_value=True)
    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.database_has_seed', return_value=True)
    def test_add_data_succesfully(self, mock_fun, mock_fun2):
        """
        Tests that the put/create method returns a 400(no content) when it gets non-json.
        :return:
        """
        body = '{"name":"Weather","data":"{\\"high\\":\\"84\\"}","timestamp":"2016-04-19 22:40:57.880853"}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        r = rest.post_seed_data("Weather")
        self.assertEquals(r["name"], "Weather")
        self.assertEquals(int(json.loads(r["data"])["high"]), 84)

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.add_data_to_database', return_value=False)
    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.database_has_seed', return_value=True)
    def test_fail_to_store(self, mock_fun, mock_fun2):
        """
        Tests that the put/create method returns a 400(no content) when it gets non-json.
        :return:
        """
        body = '{"name":"Weather","data":"{\\"high\\":\\"84\\"}","timestamp":"2016-04-19 22:40:57.880853"}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        try:
            rest.post_seed_data("Weather")
        except HTTPError as e:
            self.assertEquals(e.status_code, 500)

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.add_data_to_database', side_effect=KeyError('foo'))
    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.database_has_seed', return_value=True)
    def test_fail_to_store_error(self, mock_fun, mock_fun2):
        """
        Tests that the put/create method returns a 400(no content) when it gets non-json.
        :return:
        """
        body = '{"name":"Weather","data":"{\\"high\\":\\"84\\"}","timestamp":"2016-04-19 22:40:57.880853"}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        try:
            rest.post_seed_data("Weather")
        except HTTPError as e:
            self.assertEquals(e.status_code, 400)


class TestGetListOfNames(unittest.TestCase):
    def setUp(self):
        # Flush any cached values
        request.bind({})

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.get_seed_list', return_value=None)
    def testNoneList(self, mock_fun):
        r = rest.get_list_of_seeds()
        self.assertEquals(r.status_code, 204)

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.get_seed_list', return_value=[])
    def testEmptyList(self, mock_fun):
        r = rest.get_list_of_seeds()
        self.assertEquals(r.status_code, 204)

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.get_seed_list', return_value=["Weather", "Finance", "IOT"])
    def test_full_list(self, mock_fun):
        r = rest.get_list_of_seeds()
        self.assertEquals(len(r['names']), 3)
        self.assertEquals(r['names'], ["Weather", "Finance", "IOT"])


class TestGetData(unittest.TestCase):
    def setUp(self):
        # Flush any cached values
        request.bind({})

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.database_has_seed', return_value=False)
    def test_doesnt_exist(self, mock_fun):
        """
        Tests that the put/create method returns a 204 (no content) when it creates an empty object.
        :return:
        """
        body = '{"name":""}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        try:
            rest.get_seed_data("Weather")
        except HTTPResponse as e:
            self.assertEquals(e.status_code, 400)

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.get_seed_data', return_value=None)
    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.database_has_seed', return_value=True)
    def test_return_empty_document(self, mock_fun, mock_fun2):
        """
        Tests that the put/create method returns a 400(no content) when it gets non-json.
        :return:
        """
        body = '{}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        r = rest.get_seed_data("weather")
        self.assertEquals(r.status_code, 204)

    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.get_seed_data',
                return_value={"data": {"high": 84}, "name": "weather", "timestamp": "2016-04-19 22:40:57.880853",
                              "logtimestamp": "2016-04-19 22:41:52.235453"})
    @mock.patch('rest.DBWrapper.pepp_talk_db_wrapper.database_has_seed', return_value=True)
    def test_retrieve_data(self, mock_fun, mock_fun2):
        """
        Tests that the put/create method returns a 400(no content) when it gets non-json.
        :return:
        """
        body = '{"name":"Weather","data":"{\\"high\\":\\"84\\"}","timestamp":"2016-04-19 22:40:57.880853"}'
        request.environ['CONTENT_LENGTH'] = str(len(tob(body)))
        request.environ['CONTENT_TYPE'] = "application/json"
        request.environ['wsgi.input'] = BytesIO()
        request.environ['wsgi.input'].write(tob(body))
        request.environ['wsgi.input'].seek(0)
        r = rest.get_seed_data("weather")
        self.assertEquals(r['name'], "weather")
        self.assertEquals(int(r['data']['high']), 84)
