__author__ = 'Colin Versteeg'
from unqlite import UnQLite


class Wrapper:
    def __init__(self, db_path):
        """
        Creates a new DB Wrapper for UnQLite
        :param db_path: The Path to the Database. In the server, we use "/var/www/pepptalk/pepptalk.db"
        """
        self.db = UnQLite(db_path)

    def add_seed_to_database(self, name):
        """
        Adds a seed with a sepcific name to the database.
        :param name: the name of the seed to add.
        :return: True if the seed is added to the database, False if the seed already exists.
        """
        ret = True
        collection = self.db.collection(name)
        if collection.exists():
            ret = False
        collection.create()
        self.db.commit()
        return ret

    def add_data_to_database(self, json_dat):
        """
        Adds a dictionary of Data to the database.
        :param json_dat: A dictionary of data to be added to the database. It is added to the seed with the name
        contained in the 'name' field of the dictionary.
        :return: True if it is added, false if it is not.
        """
        collection = self.db.collection(json_dat['name'])
        try:
            collection.store(json_dat)
            self.db.commit()
        except ValueError:
            return False
        return True

    def database_has_seed(self, name):
        """
        Checks if the database has the seed.
        :param name: The seed to check if exists.
        :return: True if the seed exists, false otherwise.
        """
        collection = self.db.collection(name)
        return collection.exists()

    def get_seed_list(self):
        """
        Returns the list of seeds in the database.
        :return: The list of the names of seeds in the database.
        """
        return [ k for k in self.db.keys() if self.db.collection(k).exists()]

    def get_seed_data(self, name):
        """
        Returns the most recent data added to the seed.
        :param name: Name of the seed to retrieve data from.
        :return: The dictionary most recently added to the seed, or None if there is no data.
        """
        collection = self.db.collection(name)
        ret = None
        try:
            id = collection.last_record_id()
            ret = collection.fetch(id)
            if ret is not None and '__id' in ret:
                del ret['__id']
        except ValueError as e:
            ret = None
        return ret

    def get_seed_data_old(self, name, o_id):
        """
        Returns old data from the seed.
        :param name:  The seed to retrieve data from
        :param o_id:  The age of the data to retrieve data from. 0 retrieves the most recent data, while higher numbers retrieve older data.
        :return: The data from the seed.
        """
        collection = self.db.collection(name)
        ret = None
        try:
            id = collection.last_record_id() - o_id
            ret = collection.fetch(id)
            if ret is not None and '__id' in ret:
                del ret['__id']
        except ValueError as e:
            ret = None
        return ret

    #####Below methods aren't used in the REST API.

    def remove_seed_data(self,name,o_id):
        """
        Remvove the data associated with a specific seed and id.
        :param name: The name of the seed data to remove.
        :param o_id: The id of the data to remove.
        :return: True if it is removed, False otherwise.
        """
        collection = self.db.collection(name)
        id = collection.last_record_id() - o_id
        ret = collection.delete(id)
        self.db.commit()
        return ret

    def remove_seed(self,name):
        """
        Remove a seed.
        :param name: The name of the seed to remove.
        :return: True if the seed is removed, false otherwise.
        """
        collection = self.db.collection(name)
        ret = collection.drop()
        self.db.commit()
        return ret

    def clear_database(self):
        """
        Clear all the data in the database.
        :return: True if data is cleared, false otherwise.
        """
        self.db.flush()
        ret = self.db.commit()
        return (ret > 0)